@@ -16,8 +16,11 @@
 
 
 import datetime
+import warnings
 
-from keystoneclient.openstack.common import timeutils
+from oslo_utils import timeutils
+
+from keystoneclient.i18n import _
 from keystoneclient import service_catalog
 
 
@@ -33,35 +36,54 @@ class AccessInfo(dict):
     """
 
     @classmethod
-    def factory(cls, resp=None, body=None, region_name=None, **kwargs):
+    def factory(cls, resp=None, body=None, region_name=None, auth_token=None,
+                **kwargs):
         """Create AccessInfo object given a successful auth response & body
            or a user-provided dict.
+
+        .. warning::
+
+            Use of the region_name argument is deprecated as of the 1.7.0
+            release and may be removed in the 2.0.0 release.
+
         """
-        # FIXME(jamielennox): Passing region_name is deprecated. Provide an
-        # appropriate warning.
+
+        if region_name:
+            warnings.warn(
+                'Use of the region_name argument is deprecated as of the '
+                '1.7.0 release and may be removed in the 2.0.0 release.',
+                DeprecationWarning)
+
+        auth_ref = None
 
         if body is not None or len(kwargs):
             if AccessInfoV3.is_valid(body, **kwargs):
-                token = None
-                if resp:
-                    token = resp.headers['X-Subject-Token']
+                if resp and not auth_token:
+                    auth_token = resp.headers['X-Subject-Token']
+                # NOTE(jamielennox): these return AccessInfo because they
+                # already have auth_token installed on them.
                 if body:
                     if region_name:
                         body['token']['region_name'] = region_name
-                    return AccessInfoV3(token, **body['token'])
+                    return AccessInfoV3(auth_token, **body['token'])
                 else:
-                    return AccessInfoV3(token, **kwargs)
+                    return AccessInfoV3(auth_token, **kwargs)
             elif AccessInfoV2.is_valid(body, **kwargs):
                 if body:
                     if region_name:
                         body['access']['region_name'] = region_name
-                    return AccessInfoV2(**body['access'])
+                    auth_ref = AccessInfoV2(**body['access'])
                 else:
-                    return AccessInfoV2(**kwargs)
+                    auth_ref = AccessInfoV2(**kwargs)
             else:
-                raise NotImplementedError('Unrecognized auth response')
+                raise NotImplementedError(_('Unrecognized auth response'))
         else:
-            return AccessInfoV2(**kwargs)
+            auth_ref = AccessInfoV2(**kwargs)
+
+        if auth_token:
+            auth_ref.auth_token = auth_token
+
+        return auth_ref
 
     def __init__(self, *args, **kwargs):
         super(AccessInfo, self).__init__(*args, **kwargs)
@@ -75,7 +97,8 @@ class AccessInfo(dict):
     def will_expire_soon(self, stale_duration=None):
         """Determines if expiration is about to occur.
 
-        :return: boolean : true if expiration is within the given duration
+        :returns: true if expiration is within the given duration
+        :rtype: boolean
 
         """
         stale_duration = (STALE_TOKEN_DURATION if stale_duration is None
@@ -92,7 +115,8 @@ class AccessInfo(dict):
         """Determines if processing v2 or v3 token given a successful
         auth body or a user-provided dict.
 
-        :return: boolean : true if auth body matches implementing class
+        :returns: true if auth body matches implementing class
+        :rtype: boolean
         """
         raise NotImplementedError()
 
@@ -110,7 +134,18 @@ class AccessInfo(dict):
 
         :returns: str
         """
-        raise NotImplementedError()
+        return self['auth_token']
+
+    @auth_token.setter
+    def auth_token(self, value):
+        self['auth_token'] = value
+
+    @auth_token.deleter
+    def auth_token(self):
+        try:
+            del self['auth_token']
+        except KeyError:
+            pass
 
     @property
     def expires(self):
@@ -120,6 +155,14 @@ class AccessInfo(dict):
         """
         raise NotImplementedError()
 
+    @property
+    def issued(self):
+        """Returns the token issue time (as datetime object)
+
+        :returns: datetime
+        """
+        raise NotImplementedError()
+
     @property
     def username(self):
         """Returns the username associated with the authentication request.
@@ -163,6 +206,15 @@ class AccessInfo(dict):
         """
         raise NotImplementedError()
 
+    @property
+    def role_ids(self):
+        """Returns a list of role ids of the user associated with the
+        authentication request.
+
+        :returns: a list of strings of role ids
+        """
+        raise NotImplementedError()
+
     @property
     def role_names(self):
         """Returns a list of role names of the user associated with the
@@ -206,7 +258,10 @@ class AccessInfo(dict):
         """Returns true if the authorization token was scoped to a tenant
            (project), and contains a populated service catalog.
 
-           This is deprecated, use project_scoped instead.
+        .. warning::
+
+            This is deprecated as of the 1.7.0 release in favor of
+            project_scoped and may be removed in the 2.0.0 release.
 
         :returns: bool
         """
@@ -246,6 +301,22 @@ class AccessInfo(dict):
         """
         raise NotImplementedError()
 
+    @property
+    def trustee_user_id(self):
+        """Returns the trustee user id associated with a trust.
+
+        :returns: str or None (if no trust associated with the token)
+        """
+        raise NotImplementedError()
+
+    @property
+    def trustor_user_id(self):
+        """Returns the trustor user id associated with a trust.
+
+        :returns: str or None (if no trust associated with the token)
+        """
+        raise NotImplementedError()
+
     @property
     def project_id(self):
         """Returns the project ID associated with the authentication
@@ -293,7 +364,8 @@ class AccessInfo(dict):
         (project), this property will return None.
 
         DEPRECATED: this doesn't correctly handle region name. You should fetch
-        it from the service catalog yourself.
+        it from the service catalog yourself. This may be removed in the 2.0.0
+        release.
 
         :returns: tuple of urls
         """
@@ -306,7 +378,8 @@ class AccessInfo(dict):
         authentication request wasn't scoped to a tenant (project).
 
         DEPRECATED: this doesn't correctly handle region name. You should fetch
-        it from the service catalog yourself.
+        it from the service catalog yourself. This may be removed in the 2.0.0
+        release.
 
         :returns: tuple of urls
         """
@@ -320,6 +393,59 @@ class AccessInfo(dict):
         """
         return self.get('version')
 
+    @property
+    def oauth_access_token_id(self):
+        """Return the access token ID if OAuth authentication used.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def oauth_consumer_id(self):
+        """Return the consumer ID if OAuth authentication used.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def is_federated(self):
+        """Returns true if federation was used to get the token.
+
+        :returns: boolean
+        """
+        raise NotImplementedError()
+
+    @property
+    def audit_id(self):
+        """Return the audit ID if present.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def audit_chain_id(self):
+        """Return the audit chain ID if present.
+
+        In the event that a token was rescoped then this ID will be the
+        :py:attr:`audit_id` of the initial token. Returns None if no value
+        present.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def initial_audit_id(self):
+        """The audit ID of the initially requested token.
+
+        This is the :py:attr:`audit_chain_id` if present or the
+        :py:attr:`audit_id`.
+        """
+        return self.audit_chain_id or self.audit_id
+
 
 class AccessInfoV2(AccessInfo):
     """An object for encapsulating a raw v2 auth token from identity
@@ -346,14 +472,21 @@ class AccessInfoV2(AccessInfo):
     def has_service_catalog(self):
         return 'serviceCatalog' in self
 
-    @property
+    @AccessInfo.auth_token.getter
     def auth_token(self):
-        return self['token']['id']
+        try:
+            return super(AccessInfoV2, self).auth_token
+        except KeyError:
+            return self['token']['id']
 
     @property
     def expires(self):
         return timeutils.parse_isotime(self['token']['expires'])
 
+    @property
+    def issued(self):
+        return timeutils.parse_isotime(self['token']['issued_at'])
+
     @property
     def username(self):
         return self['user'].get('name', self['user'].get('username'))
@@ -370,6 +503,10 @@ class AccessInfoV2(AccessInfo):
     def user_domain_name(self):
         return 'Default'
 
+    @property
+    def role_ids(self):
+        return self.get('metadata', {}).get('roles', [])
+
     @property
     def role_names(self):
         return [r['name'] for r in self['user'].get('roles', [])]
@@ -405,6 +542,13 @@ class AccessInfoV2(AccessInfo):
 
     @property
     def scoped(self):
+        """Deprecated as of the 1.7.0 release in favor of project_scoped and
+        may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'scoped is deprecated as of the 1.7.0 release in favor of '
+            'project_scoped and may be removed in the 2.0.0 release.',
+            DeprecationWarning)
         if ('serviceCatalog' in self
                 and self['serviceCatalog']
                 and 'tenant' in self['token']):
@@ -427,6 +571,15 @@ class AccessInfoV2(AccessInfo):
     def trust_scoped(self):
         return 'trust' in self
 
+    @property
+    def trustee_user_id(self):
+        return self.get('trust', {}).get('trustee_user_id')
+
+    @property
+    def trustor_user_id(self):
+        # this information is not available in the v2 token bug: #1331882
+        return None
+
     @property
     def project_id(self):
         try:
@@ -460,8 +613,13 @@ class AccessInfoV2(AccessInfo):
 
     @property
     def auth_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release in favor of
+        service_catalog.get_urls() and may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'auth_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='publicURL',
@@ -471,8 +629,13 @@ class AccessInfoV2(AccessInfo):
 
     @property
     def management_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release in favor of
+        service_catalog.get_urls() and may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'management_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='adminURL',
@@ -480,6 +643,32 @@ class AccessInfoV2(AccessInfo):
         else:
             return None
 
+    @property
+    def oauth_access_token_id(self):
+        return None
+
+    @property
+    def oauth_consumer_id(self):
+        return None
+
+    @property
+    def is_federated(self):
+        return False
+
+    @property
+    def audit_id(self):
+        try:
+            return self['token'].get('audit_ids', [])[0]
+        except IndexError:
+            return None
+
+    @property
+    def audit_chain_id(self):
+        try:
+            return self['token'].get('audit_ids', [])[1]
+        except IndexError:
+            return None
+
 
 class AccessInfoV3(AccessInfo):
     """An object for encapsulating a raw v3 auth token from identity
@@ -494,7 +683,7 @@ class AccessInfoV3(AccessInfo):
             token=token,
             region_name=self._region_name)
         if token:
-            self.update(auth_token=token)
+            self.auth_token = token
 
     @classmethod
     def is_valid(cls, body, **kwargs):
@@ -509,24 +698,42 @@ class AccessInfoV3(AccessInfo):
         return 'catalog' in self
 
     @property
-    def auth_token(self):
-        return self['auth_token']
+    def is_federated(self):
+        return 'OS-FEDERATION' in self['user']
 
     @property
     def expires(self):
         return timeutils.parse_isotime(self['expires_at'])
 
+    @property
+    def issued(self):
+        return timeutils.parse_isotime(self['issued_at'])
+
     @property
     def user_id(self):
         return self['user']['id']
 
     @property
     def user_domain_id(self):
-        return self['user']['domain']['id']
+        try:
+            return self['user']['domain']['id']
+        except KeyError:
+            if self.is_federated:
+                return None
+            raise
 
     @property
     def user_domain_name(self):
-        return self['user']['domain']['name']
+        try:
+            return self['user']['domain']['name']
+        except KeyError:
+            if self.is_federated:
+                return None
+            raise
+
+    @property
+    def role_ids(self):
+        return [r['id'] for r in self.get('roles', [])]
 
     @property
     def role_names(self):
@@ -574,6 +781,13 @@ class AccessInfoV3(AccessInfo):
 
     @property
     def scoped(self):
+        """Deprecated as of the 1.7.0 release in favor of project_scoped and
+        may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'scoped is deprecated as of the 1.7.0 release in favor of '
+            'project_scoped and may be removed in the 2.0.0 release.',
+            DeprecationWarning)
         return ('catalog' in self and self['catalog'] and 'project' in self)
 
     @property
@@ -592,10 +806,23 @@ class AccessInfoV3(AccessInfo):
     def trust_scoped(self):
         return 'OS-TRUST:trust' in self
 
+    @property
+    def trustee_user_id(self):
+        return self.get('OS-TRUST:trust', {}).get('trustee_user', {}).get('id')
+
+    @property
+    def trustor_user_id(self):
+        return self.get('OS-TRUST:trust', {}).get('trustor_user', {}).get('id')
+
     @property
     def auth_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release in favor of
+        service_catalog.get_urls() and may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'auth_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='public',
@@ -605,8 +832,13 @@ class AccessInfoV3(AccessInfo):
 
     @property
     def management_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release in favor of
+        service_catalog.get_urls() and may be removed in the 2.0.0 release.
+        """
+        warnings.warn(
+            'management_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='admin',
@@ -614,3 +846,25 @@ class AccessInfoV3(AccessInfo):
 
         else:
             return None
+
+    @property
+    def oauth_access_token_id(self):
+        return self.get('OS-OAUTH1', {}).get('access_token_id')
+
+    @property
+    def oauth_consumer_id(self):
+        return self.get('OS-OAUTH1', {}).get('consumer_id')
+
+    @property
+    def audit_id(self):
+        try:
+            return self.get('audit_ids', [])[0]
+        except IndexError:
+            return None
+
+    @property
+    def audit_chain_id(self):
+        try:
+            return self.get('audit_ids', [])[1]
+        except IndexError:
+            return None
