@@ -16,6 +16,12 @@
 """
 TOKEN-BASED AUTH MIDDLEWARE
 
+.. warning::
+
+  This module is DEPRECATED. The auth_token middleware has been moved to the
+  `keystonemiddleware repository
+  <http://docs.openstack.org/developer/keystonemiddleware/>`_.
+
 This WSGI component:
 
 * Verifies that incoming client requests have valid tokens by validating
@@ -26,9 +32,6 @@ This WSGI component:
 * Collects and forwards identity information based on a valid token
   such as user name, tenant, etc
 
-Refer to: http://docs.openstack.org/developer/python-keystoneclient/
-middlewarearchitecture.html
-
 HEADERS
 -------
 
@@ -154,7 +157,9 @@ import tempfile
 import time
 
 import netaddr
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 import requests
 import six
 from six.moves import urllib
@@ -163,9 +168,8 @@ from keystoneclient import access
 from keystoneclient.common import cms
 from keystoneclient import exceptions
 from keystoneclient.middleware import memcache_crypt
-from keystoneclient.openstack.common import jsonutils
 from keystoneclient.openstack.common import memorycache
-from keystoneclient.openstack.common import timeutils
+from keystoneclient import utils
 
 
 # alternative middleware configuration in the main application's
@@ -339,7 +343,7 @@ LIST_OF_VERSIONS_TO_ATTEMPT = ['v2.0', 'v3.0']
 CACHE_KEY_TEMPLATE = 'tokens/%s'
 
 
-class BIND_MODE:
+class BIND_MODE(object):
     DISABLED = 'disabled'
     PERMISSIVE = 'permissive'
     STRICT = 'strict'
@@ -379,7 +383,7 @@ def confirm_token_not_expired(data):
     utcnow = timeutils.utcnow()
     if utcnow >= expires:
         raise InvalidUserToken('Token authorization failed')
-    return timeutils.isotime(at=expires, subsecond=True)
+    return utils.isotime(at=expires, subsecond=True)
 
 
 def _v3_to_v2_catalog(catalog):
@@ -423,6 +427,27 @@ def safe_quote(s):
     return urllib.parse.quote(s) if s == urllib.parse.unquote(s) else s
 
 
+def _conf_values_type_convert(conf):
+    """Convert conf values into correct type."""
+    if not conf:
+        return {}
+    _opts = {}
+    opt_types = dict((o.dest, getattr(o, 'type', str)) for o in opts)
+    for k, v in six.iteritems(conf):
+        try:
+            if v is None:
+                _opts[k] = v
+            else:
+                _opts[k] = opt_types[k](v)
+        except KeyError:
+            _opts[k] = v
+        except ValueError as e:
+            raise ConfigurationError(
+                'Unable to convert the value of %s option into correct '
+                'type: %s' % (k, e))
+    return _opts
+
+
 class InvalidUserToken(Exception):
     pass
 
@@ -462,7 +487,10 @@ class AuthProtocol(object):
             'This middleware module is deprecated as of v0.10.0 in favor of '
             'keystonemiddleware.auth_token - please update your WSGI pipeline '
             'to reference the new middleware package.')
-        self.conf = conf
+        # NOTE(wanghong): If options are set in paste file, all the option
+        # values passed into conf are string type. So, we should convert the
+        # conf value into correct type.
+        self.conf = _conf_values_type_convert(conf)
         self.app = app
 
         # delay_auth_decision means we still allow unauthenticated requests
@@ -723,8 +751,8 @@ class AuthProtocol(object):
             return token
         else:
             if not self.delay_auth_decision:
-                self.LOG.warn('Unable to find authentication token'
-                              ' in headers')
+                self.LOG.warning('Unable to find authentication token'
+                                 ' in headers')
                 self.LOG.debug('Headers: %s', env)
             raise InvalidUserToken('Unable to find token in headers')
 
@@ -777,8 +805,8 @@ class AuthProtocol(object):
         if self.cert_file and self.key_file:
             kwargs['cert'] = (self.cert_file, self.key_file)
         elif self.cert_file or self.key_file:
-            self.LOG.warn('Cannot use only a cert or key file. '
-                          'Please provide both. Ignoring.')
+            self.LOG.warning('Cannot use only a cert or key file. '
+                             'Please provide both. Ignoring.')
 
         kwargs['verify'] = self.ssl_ca_file or True
         if self.ssl_insecure:
@@ -795,7 +823,8 @@ class AuthProtocol(object):
                     self.LOG.error('HTTP connection exception: %s', e)
                     raise NetworkError('Unable to communicate with keystone')
                 # NOTE(vish): sleep 0.5, 1, 2
-                self.LOG.warn('Retrying on HTTP connection exception: %s', e)
+                self.LOG.warning('Retrying on HTTP connection exception: %s',
+                                 e)
                 time.sleep(2.0 ** retry / 2)
                 retry += 1
 
@@ -869,12 +898,12 @@ class AuthProtocol(object):
             datetime_expiry = timeutils.parse_isotime(expiry)
             return (token, timeutils.normalize_time(datetime_expiry))
         except (AssertionError, KeyError):
-            self.LOG.warn(
+            self.LOG.warning(
                 'Unexpected response from keystone service: %s', data)
             raise ServiceError('invalid json response')
         except (ValueError):
             data['access']['token']['id'] = '<SANITIZED>'
-            self.LOG.warn(
+            self.LOG.warning(
                 'Unable to parse expiration time from token: %s', data)
             raise ServiceError('invalid json response')
 
@@ -921,13 +950,13 @@ class AuthProtocol(object):
             return data
         except NetworkError:
             self.LOG.debug('Token validation failure.', exc_info=True)
-            self.LOG.warn('Authorization failed for token')
+            self.LOG.warning('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
         except Exception:
             self.LOG.debug('Token validation failure.', exc_info=True)
             if token_id:
                 self._token_cache.store_invalid(token_id)
-            self.LOG.warn('Authorization failed for token')
+            self.LOG.warning('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
 
     def _build_user_headers(self, token_info):
@@ -1086,7 +1115,7 @@ class AuthProtocol(object):
         :param retry: flag that forces the middleware to retry
                       user authentication when an indeterminate
                       response is received. Optional.
-        :return: token object received from keystone on success
+        :returns: token object received from keystone on success
         :raise InvalidUserToken: if token is rejected
         :raise ServiceError: if unable to authenticate token
 
@@ -1116,7 +1145,7 @@ class AuthProtocol(object):
         if response.status_code == 200:
             return data
         if response.status_code == 404:
-            self.LOG.warn('Authorization failed for token')
+            self.LOG.warning('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
         if response.status_code == 401:
             self.LOG.info(
@@ -1129,7 +1158,7 @@ class AuthProtocol(object):
             self.LOG.info('Retrying validation')
             return self.verify_uuid_token(user_token, False)
         else:
-            self.LOG.warn('Invalid user token. Keystone response: %s', data)
+            self.LOG.warning('Invalid user token. Keystone response: %s', data)
 
             raise InvalidUserToken()
 
@@ -1407,7 +1436,7 @@ class TokenCache(object):
 
         """
 
-        if cms.is_asn1_token(user_token):
+        if cms.is_asn1_token(user_token) or cms.is_pkiz(user_token):
             # user_token is a PKI token that's not hashed.
 
             token_hashes = list(cms.cms_hash_token(user_token, mode=algo)
